import Taro from '@tarojs/taro'

export const WX_AB_GROUP = { ab_test010: 'ab100'}
for (let i = 1; i < 10; i++)
  WX_AB_GROUP[`ab_test00${i}`] = 'ab100'

class AbGroupContext implements AbGroupContextType {
  #group: object = WX_AB_GROUP
  #evtStack: Array<Function> = []
  #timer: NodeJS.Timeout | null
  #groupMap = new Map()

  constructor() {
    this.#group = {
      ...this.#group,
      ...Taro.getExptInfoSync()
    }
  }

  listen(evt) {
    if (typeof evt !== 'function')
      return this

    this.#evtStack.push(evt)

    return this
  }

  emit() {
    this.#evtStack.forEach(evt => evt())

    return this
  }

  loop(time: number = 60000) {
    if (this.#timer)
      return this

    this.#timer = setInterval(() => {
      this.emit()
    }, time)

    return this

  }

  cleanLoop() {
    if (this.#timer) 
      clearInterval(this.#timer)

    return this
  }

  cleanStack() {
    this.#evtStack = []

    return this
  }

  get() {
    return this.#group
  }

  storeGroup(config) {
    Object.keys(config).forEach(level => {
      const abGroupConfigArray = config[level][this.#group[level]] || []
      abGroupConfigArray.forEach(item => {
        this.#groupMap.set(item.abExpCode, {
          level: level,
          group: this.#group[level],
          experiment: item.abExpCode,
          config: item.configValue
        })
      })
    })

    this.setStorage(config)
  }

  validExperiment(name) {
    return this.#groupMap.has(name + '')
  }

  getExperimentConfigByName(name) {
    return this.#groupMap.get(name + '')
  }

  setStorage(config) {
    const EXPERIMENT_CONFIG = {}
    Object.keys(config).forEach(level => {
      const abGroupConfigArray = config[level][this.#group[level]] || []
      EXPERIMENT_CONFIG[level] = abGroupConfigArray
    })
    Taro.setStorageSync('$EXPERIMENT_CONFIG', EXPERIMENT_CONFIG)
  }

}

export default AbGroupContext
