import Taro from '@tarojs/taro'

export function isEasyMode(systemInfo: Taro.getSystemInfoSync.Result) {
  const { fontSizeSetting = 0 } = systemInfo

  if (systemInfo.platform.indexOf('ios') !== -1)
    return fontSizeSetting > 16

  if (systemInfo.platform.indexOf('Android') !== -1)
    return fontSizeSetting > 17

  return false
}

export function S4() {
  return (((1 + Math.random()) * 0x10000) | 0).toString(16).substring(1)
}

export function guid() {
  return S4() + S4() + '-' + S4() + '-' + S4() + '-' + S4() + '-' + S4() + S4() + S4()
}

function sessionGuid() {
  return `${new Date().getTime()}-${guid()}`
}

export function getPreIds() {
  return 'pre-' + sessionGuid()
}

export function getViewIds() {
  return 'view-' + sessionGuid()
}

export function createSessionId() {
  return `${new Date().getTime()}-${guid()}`
}

export function getTopSafeHeight() {
  // 状态栏 刘海屏
  const systemInfo = Taro.getSystemInfoSync()
  const { statusBarHeight = 0 } = systemInfo
  // 胶囊信息
  const menuInfo = Taro.getMenuButtonBoundingClientRect()
  // 导航高度固定 44px
  let navigationBarHeight = (menuInfo.top - statusBarHeight) * 2 + menuInfo.height

  navigationBarHeight = Math.min(44, navigationBarHeight)

  return {
    navigationBarHeight,
    statusBarHeight
  }
}

export function throttle(fn, wait = 50) {
  let previous = 0
  return function(...args) {
    let now = +new Date()
    if (now - previous > wait) {
      previous = now
      fn.apply(this, args)
    }
  }
}

export function formatSecondsAsTime(secs) {
  var hr = Math.floor(secs / 3600) + ''
  var min = Math.floor((secs - +hr * 3600) / 60) + ''
  var sec = Math.floor(secs - +hr * 3600 - +min * 60) + ''
  var text
  if (+hr < 10) {
      hr = '0' + hr
  }
  if (+min < 10) {
      min = '0' + min
  }
  if (+sec < 10) {
      sec = '0' + sec
  }
  if (+hr == 0) {
      hr = ''
  }
  if (hr.length > 0) {
      text = hr + ':' + min + ':' + sec
  } else {
      text = min + ':' + sec
  }
  return text
}

export function type(object, type) {
  return Object.prototype.toString.call(object) === `[object ${type}]`
}

export function stringify(obj) {
  if (!type(obj, 'Object'))
    return ''

  return Object.entries(obj).reduce((calc, [key, val], index) => {
    calc += `${key}=${val}`

    if (index !== Object.entries(obj).length - 1)
      calc += '&'

    return calc
  }, '')
}

export const squaredImageUrl = (originalURL, rotate, width) => {
  // 正方形: rotate 旋转角度(整型),  方形图片
  var url = originalURL
  var idx = originalURL.indexOf('?')
  if (idx !== -1) {
      // 防止该URL已经拼接过参数
      url = originalURL.substringToIndex(idx)
  }
  return url + '?x-oss-process=image/rotate,' + Math.ceil(rotate) + '/resize,m_fill,w_' + Math.ceil(width) + ',h_' + Math.ceil(width) + ',limit_0' + '/format,jpg'
}

export function filterCoverPath(path) {
  if (path && path.indexOf('?') === -1) {
      path += `?t=${Date.now()}`
  }
  path = path.replace(/resupload\.piaoquantv\.com/g, 'rescdn.yishihui.com')
  path = path.replace(/weappupload\.piaoquantv\.com/g, 'rescdn.yishihui.com')

  return path
}

export function mediaRotate(orientation) {
  switch (orientation) {
    case 'down':
        return 180

    case 'left':
        return -90

    case 'right':
        return 90

    case 'up-mirrored':
        return 0

    case 'down-mirrored':
        return 180

    case 'left-mirrored':
        return -90

    case 'right-mirrored':
        return 90

    default:
        return 0
  }
}

export function once(cb) {
  let count = 1

  return (...args) => {
    if (count++ > 1)
      return

    cb.apply(null, args)
  }
}

export function formatQuery(options) {
  if (Object.prototype.toString.call(options) !== '[object Object]')
    return options

  const orgOptions = options
  const params = {}

  let jumpPage = ''

  Object.keys(options).forEach((key) => {
    try {
        const decodeVal = decodeURIComponent(options[key])
        
        params[key] = decodeVal

        if (key === 'jumpPage')
            jumpPage = decodeVal

    } catch (e) {}
  })

  return {
    params,
    jumpPage,
    orgOptions
  }
}

export function formatDate(date, fmt) {
  var o = {
      'M+': date.getMonth() + 1, //月份
      'd+': date.getDate(), //日
      'h+': date.getHours(), //小时
      'm+': date.getMinutes(), //分
      's+': date.getSeconds(), //秒
      'q+': Math.floor((date.getMonth() + 3) / 3), //季度
      S: date.getMilliseconds() //毫秒
  }
  if (/(y+)/.test(fmt)) {
      fmt = fmt.replace(RegExp.$1, (date.getFullYear() + '').substr(4 - RegExp.$1.length))
  }
  for (var k in o) {
      if (new RegExp('(' + k + ')').test(fmt)) {
          fmt = fmt.replace(RegExp.$1, RegExp.$1.length == 1 ? o[k] : ('00' + o[k]).substr(('' + o[k]).length))
      }
  }
  return fmt
}